@@ -54,13 +54,8 @@ namespace ost {
 char AculabTrunk::status[MAXPORT * MAXTIMESLOTS];
 
 AculabTrunk::AculabTrunk(int p, int t) :
-#ifdef	COMMON_OST_NAMESPACE
-Trunk(p * MAXTIMESLOTS + t), TimerPort(), URLAudio(), AudioService()
-#else
-Trunk(p * MAXTIMESLOTS + t), TimerPort(), URLAudio(), AudioService()
-#endif
+Trunk(p * MAXTIMESLOTS + t), TimerPort(), AudioService(), URLAudio()
 {
-	unsigned long opts;
 	struct in_xparms in_xparms;
 	int rc;
 
@@ -140,6 +135,7 @@ void AculabTrunk::exit(void)
 	if(!Trunk::flags.onexit)
 		if(redirect("::exit"))
 		{
+			autoloop(false);
 			Trunk::flags.onexit = true;
 			return;
 		}
@@ -233,9 +229,12 @@ retry:
                         break;
                 }
 
-		trunkSignal(TRUNK_SIGNAL_HANGUP);
-		event->id = TRUNK_STOP_STATE;
-		goto retry;
+		if(trunkSignal(TRUNK_SIGNAL_HANGUP))
+		{
+			event->id = TRUNK_STOP_STATE;
+			goto retry;
+		}
+		break;
 	case TRUNK_TIMER_EXPIRED:
 		trunkSignal(TRUNK_SIGNAL_TIMEOUT);
 		event->id = TRUNK_STOP_STATE;
@@ -343,13 +342,36 @@ long AculabTrunk::getMaxTime(void)
 bool AculabTrunk::scrJoin(void)
 {
 	Trunk *trunk = aculabivr.getTrunkId(getValue(getKeyword("id")));
+	const char *mem = getMember();
 
-	if(!trunk) {
+	if(!trunk) 
+	{
 		Trunk::error("join-no-port");
 		return true;
 	}
 
-	data.join.trunk = trunk;
+	if(!mem)
+		mem = "duplex";
+
+	data.join.hangup = false;
+	data.join.waiting = NULL;
+
+	if(!stricmp(mem, "hangup"))
+		data.join.hangup = true;
+
+	mem = getKeyword("count");
+	if(mem)
+		data.join.count = atoi(mem);
+	else
+		data.join.count = 0;
+
+	mem = getKeyword("waitTime");
+	if(mem)
+               data.join.count = getSecTimeout(mem) /
+                        keythreads.getResetDelay() + 1;
+
+	data.join.waiting = data.join.trunk = trunk;
+	data.join.seq = trunk->getSequence();
 	data.join.wakeup = getTimeout("maxTime");
 	trunkStep(TRUNK_STEP_JOIN);
 	return false;
@@ -357,6 +379,23 @@ bool AculabTrunk::scrJoin(void)
 
 bool AculabTrunk::scrWait(void)
 {
+	const char *mem = getMember();
+
+	if(mem)
+	{
+		data.join.waiting = NULL;
+		data.join.hangup = false;
+		if(!stricmp(mem, "hangup"))
+			data.join.hangup = true;
+		else if(!stricmp(mem, "parent"))
+			data.join.waiting = aculabivr.getTrunkId(getSymbol(SYM_PARENT));
+	}
+	else
+	{
+		data.join.waiting = NULL;
+		data.join.hangup = false;
+	}
+	data.join.count = 0;
 	data.join.trunk = NULL;
 	data.join.wakeup = getTimeout("maxTime");
 	trunkStep(TRUNK_STEP_JOIN);
