@@ -140,6 +140,47 @@ bool PhonedevTrunk::sleepHandler(TrunkEvent *event)
 	return false;
 }
 
+bool PhonedevTrunk::threadHandler(TrunkEvent *event)
+{
+	switch(event->id)
+	{
+        case TRUNK_TIMER_EXPIRED:
+                stopServices();
+                endTimer();
+                setSymbol(SYM_ERROR, "thread-timeout");
+                if(!trunkSignal(TRUNK_SIGNAL_TIMEOUT))
+                        trunkSignal(TRUNK_SIGNAL_ERROR);
+                handler = &PhonedevTrunk::stepHandler;
+                return true;
+        case TRUNK_SERVICE_FAILURE:
+                flags.reset = true;
+                endTimer();
+                setSymbol(SYM_ERROR, "thread-error");
+                trunkSignal(TRUNK_SIGNAL_ERROR);
+                handler = &PhonedevTrunk::stepHandler;
+                return true;
+        case TRUNK_STOP_STATE:
+                stopServices();
+                endTimer();
+                handler = &PhonedevTrunk::stepHandler;
+                return true;
+        case TRUNK_SERVICE_SUCCESS:
+                flags.reset = true;
+		endTimer();
+		trunkSignal(TRUNK_SIGNAL_STEP);
+                handler = &PhonedevTrunk::stepHandler;
+                return true;
+        case TRUNK_ENTER_STATE:
+                enterState("thread");
+                endTimer();
+                Trunk::setDTMFDetect();
+                setTimer(data.sleep.wakeup);
+                thread->start();
+                return true;
+	}
+	return false;
+}
+
 #ifdef	CCXX_NAMESPACES
 };
 #endif
