@@ -49,9 +49,14 @@ bool PikaTrunk::hangupHandler(TrunkEvent *event)
 
 	switch(event->id)
 	{
+	case TRUNK_LINE_WINK:
+	case TRUNK_STOP_DISCONNECT:
+	case TRUNK_CPA_DIALTONE:
+		return true;
 	case TRUNK_ENTER_STATE:
 		debug->DebugState(this, "hangup");
 		status[id] = 'h';
+		disjoin();
 		endTimer();
 		if(tgi.pid)
 			kill(tgi.pid, SIGHUP);
