@@ -62,6 +62,7 @@ bool PikaTrunk::idleHandler(TrunkEvent *event)
 	case TRUNK_ENTER_STATE:
 		debug->DebugState(this, "idle");
 		ScriptSymbol::Purge();
+		disjoin();
 		stopDSP();
 		endTimer();
 
@@ -117,7 +118,7 @@ bool PikaTrunk::idleHandler(TrunkEvent *event)
 				setList(argv);
 				endTimer();
 				flags.ready = false;
-				handler = &PikaTrunk::stepHandler;
+				handler = &PikaTrunk::siezeHandler;
 				group->incOutgoing();
 				delete req;
 				return true;
@@ -130,9 +131,6 @@ bool PikaTrunk::idleHandler(TrunkEvent *event)
 		return true;
 	case TRUNK_START_SCRIPT:
 		flags.trunk = TRUNK_MODE_OUTGOING;
-
-// Matt B -- code split from TRUNK_RING_START to keep the flow clear
-	     
 		start = event->parm.argv[0];
 		++argc;
 		
@@ -149,11 +147,7 @@ bool PikaTrunk::idleHandler(TrunkEvent *event)
 		
 		  endTimer();
 		  flags.ready = false;
-		  // Looks like following call needed to start script executing?
-		  // Maybe should be stepHandler?
-		  handler = &PikaTrunk::stepHandler;
-		  //		  handler = &PikaTrunk::seizeHandler;  // Does what?
-
+		  handler = &PikaTrunk::siezeHandler;
 		  group->incOutgoing();
 			
 		  return true;
@@ -163,6 +157,22 @@ bool PikaTrunk::idleHandler(TrunkEvent *event)
 		slog(SLOG_ERROR) << pikaivr.getCardName(card) << id << ": " << start << ": cannot start" << endl;
 		return true;		
 // Matt B
+	case TRUNK_RING_REDIRECT:
+		start = group->getRedirect(event->parm.argv[0], script);
+		rings = 0;
+		if(Attach(start))
+		{
+			setConstant(SYM_REDIRECT, event->parm.argv[0]);
+			setList(&event->parm.argv[1]);
+			flags.ready = false;
+			flags.trunk = TRUNK_MODE_INCOMING;
+			endTimer();
+			handler = &PikaTrunk::stepHandler;
+			group->incIncoming();		
+			return true;
+		}
+		slog(SLOG_ERROR) << pikaivr.getCardName(card) << id << ": " << event->parm.argv[0] << ": unable to redirect" << endl;
+		return true;
 	case TRUNK_RING_START:
 		start = event->parm.argv[0];
 		if(start)
@@ -197,6 +207,8 @@ bool PikaTrunk::idleHandler(TrunkEvent *event)
 		slog(SLOG_ERROR) << pikaivr.getCardName(card) << id << ": " << start << ": cannot start" << endl;
 		return true;		
 	case TRUNK_RINGING_ON:
+		if(!group->getAnswer())
+			return true;
 		rings = 1;
 		group->incIncoming();
 		flags.ready = false;
