@@ -1,17 +1,17 @@
 // Copyright (C) 2000 Open Source Telecom Corporation.
-//  
+//
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 2 of the License, or
 // (at your option) any later version.
-// 
+//
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software 
+// along with this program; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 #include "driver.h"
@@ -21,6 +21,8 @@ using namespace std;
 namespace ost {
 #endif
 
+static	int	dropflag = false;
+
 VPBConfig::VPBConfig() :
 Keydata("/bayonne/vpb")
 {
@@ -31,10 +33,30 @@ Keydata("/bayonne/vpb")
 	{"firmware", "/etc/vpb/vpbmain_pci.out"},
 	{"model", "v4pci"},
 	{"delay", "10"},
+	{"stations", "aann"},
 	{"pickup", "800"},
 	{NULL, NULL}};
 
+	const char *cp;
+
+	if(isFHS())
+		load("/drivers/vpb");
+
 	load(defkeys);
+
+	cp = getLast("steptimer");
+	if(cp)
+		stepTimer = getMSTimeout(cp);
+	else
+		stepTimer = keythreads.getStepDelay();
+
+	cp = getLast("synctimer");
+	if(cp)
+		syncTimer = getMSTimeout(cp);
+	else
+		syncTimer = keythreads.getResetDelay();
+
+	slog(Slog::levelDebug) << "vpb timers: step=" << stepTimer << ", sync=" << syncTimer << endl;
 }
 
 size_t VPBConfig::getStack(void)
@@ -50,10 +72,21 @@ if(!cp)
 int	VPBConfig::getModel(void)
 {
 	const char *cp = getLast("model");
+	const char *dn = plugins.getDriverName();
+
+	if(dn)
+	{
+		if(!strnicmp(dn, "openline", 8))
+			cp = "V4PCI";
+#ifdef	VPB_V12PCI
+		else if(!strnicmp(dn, "openswitch", 10))
+			cp = "V12PCI";
+#endif
+	}
 
 	if(!cp)
 		cp = getenv("VPB_MODEL");
-	
+
 	if(!cp)
 		return VPB_V4PCI;
 
@@ -91,18 +124,51 @@ VPBDriver::VPBDriver() :
 Driver(), Thread(keythreads.priService() - 1)
 {
         static Script::Define keywords[] = {
-                {"join", (Method)&VPBTrunk::scrJoin, 
-			&ScriptCommand::chkHasArgs},                
-		{"wait", (Method)&VPBTrunk::scrWait, 
-			&ScriptCommand::chkHasArgs},                
+                {"join", (Method)&VPBTrunk::scrJoin,
+			&ScriptCommand::chkIgnore},
+		{"wait", (Method)&VPBTrunk::scrWait,
+			&ScriptCommand::chkIgnore},
+		{"ring", (Method)&VPBTrunk::scrRing,
+			&ScriptCommand::chkIgnore},
+		{"pickup", (Method)&VPBTrunk::scrPickup,
+			&ScriptCommand::chkIgnore},
+		{"answer", (Method)&VPBTrunk::scrAnswer,
+			&ScriptCommand::chkIgnore},
+		{"dial", (Method)&VPBTrunk::scrDial,
+			&ScriptCommand::chkHasArgs},
 		{NULL, NULL, NULL}};
 
 	unsigned sizeindex = getCardSize();
+	unsigned cardcount = getCards();
+	const char *stations = VPBConfig::getLast("stations");
 
-	port_count = getCards() * sizeindex;
-	bridge_count = getCards() * sizeindex / 2;
+	port_count = cardcount * sizeindex;
+	bridge_count = cardcount * sizeindex / 2;
 	status = VPBTrunk::status;
 
+	if(sizeindex > 4)
+		dropflag = true;
+
+	if(sizeindex != 12)
+		cardcount = 0;
+
+	while(cardcount--)
+	{
+		switch(*stations)
+		{
+		case 'a':
+		case 'A':
+			extCount += 8;
+			break;
+		case 'b':
+		case 'B':
+			extCount += 4;
+			break;
+		}
+		++stations;
+	}
+
+	trunks = new VPBTrunk *[port_count];
 	ports = new VPBTrunk *[port_count];
 	groups = new TrunkGroup *[port_count];
 	bridges = new bool[bridge_count];
@@ -122,11 +188,20 @@ Driver(), Thread(keythreads.priService() - 1)
 	ScriptCommand::load(keywords);
 }
 
+unsigned VPBDriver::getCaps(void)
+{
+	if(extCount)
+		return capDaemon | capSwitch | capJoin | capPSTN;
+
+	return capDaemon | capPSTN | capJoin;
+}
+
 unsigned VPBDriver::getBridge(int card)
 {
-	int i, first = (card - 1) * 2;
+	int bs = getCardSize() / 2;
+	int i, first = (card - 1) * bs;
 
-	for(i = 0; i < 2; ++i)
+	for(i = 0; i < bs; ++i)
 		if(!vpbivr.bridges[first + i])
 		{
 			vpbivr.bridges[first + i] = true;
@@ -154,20 +229,63 @@ VPBDriver::~VPBDriver()
 
 	if(groups)
 		delete[] groups;
+
+	if(trunks)
+		delete[] trunks;
+}
+
+Trunk *VPBDriver::getTrkNumber(const char *trk)
+{
+	VPBTrunk *trunk;
+	unsigned tid;
+
+	if(!trk)
+		trk = "9999";
+
+	if(!trunks || *trk == '-')
+	{
+		trunk = (VPBTrunk *)getTrunkId(trk);
+		if(trunk)
+			if(trunk->isStation)
+				trunk = NULL;
+		return (Trunk *)trunk;
+	}
+
+	tid = atoi(trk);
+	if(tid < 0 || tid >= trkCount)
+		return NULL;
+	return trunks[tid];
 }
 
 int VPBDriver::start(void)
 {
+	VPBTrunk *trunk;
+	TrunkGroup *grp;
 	unsigned sizeindex = getCardSize();
-	int count, ts;
-	int cards = getCards();
+	unsigned count, ts;
+	unsigned cards = getCards();
 	int offsets[16];
 	int first = getOffset();
+	const char *stations = VPBConfig::getLast("stations");
+	bool sta;
+	char extbuf[3];
+	int extnum = 0;
+	char *dn = plugins.getDriverName();
+
+	if(!extCount)
+		stations = "nnnnnnnn";
+	else if(extCount < 9 && !numbering)
+		setExtNumbering(1);
+	else if(!numbering)
+	{
+		extnum = 10;
+		setExtNumbering(2);
+	}
 
 	// assume from port 0x3x0 upward
 
 	for(count = 0; count < cards; ++count)
-		offsets[count] = first + 16 * count; 
+		offsets[count] = first + 16 * count;
 
 	if(active)
 	{
@@ -179,19 +297,67 @@ int VPBDriver::start(void)
 	if(getFirmware())
 		setenv("VPB_FIRMWARE", getFirmware(), 0);
 
-	if(VPBConfig::getLast("model"))
+	if(!strnicmp(dn, "openline", 8))
+		setenv("VPB_MODEL", "V4PCI", 0);
+	else if(!strnicmp(dn, "openswitch", 10))
+		setenv("VPB_MODEL", "V12PCI", 0);
+	else if(VPBConfig::getLast("model"))
 		setenv("VPB_MODEL", VPBConfig::getLast("model"), 0);
 
 	vpb_seterrormode(VPB_ERROR_CODE);
 
 	for(count = 0; count < cards; ++count)
+	{
 		for(ts = 0; ts < sizeindex; ++ts)
-			ports[count * sizeindex + ts] = new VPBTrunk(count + 1, ts);
+		{
+			sta = false;
+			switch(*stations)
+			{
+			case 'a':
+			case 'A':
+				if(ts < 8)
+					sta = true;
+				break;
+			case 'b':
+			case 'B':
+				if(ts > 7)
+					sta = true;
+				break;
+			}
+			grp = NULL;
+			if(sta)
+			{
+				if(stacards[count + 1])
+					grp = stacards[count + 1];
+				else
+					grp = stacards[0];
+			}
+			if(grp)
+				groups[count * sizeindex + ts] = grp;
+
+			ports[count * sizeindex + ts] = trunk = new VPBTrunk(count + 1, ts, sta);
+			if(!sta)
+			{
+				trunks[trkCount++] = trunk;
+				continue;
+			}
+
+			if(!numbering)
+			{
+				sprintf(extbuf, "%d", extnum++);
+				setExtNumber(count * sizeindex + ts, extbuf);
+			}
+		}
+		++stations;
+	}
 
 	if(!running)
 		Thread::start();
 
-	slog(Slog::levelInfo) << "vpb: initalized " << cards * sizeindex << " ports successfully..." << endl;
+	if(extCount)
+		slog(Slog::levelInfo) << "vpb: initialized; extensions=" << extCount << ", trunks=" << trkCount << endl;
+	else
+		slog(Slog::levelInfo) << "vpb: initalized " << cards * sizeindex << " ports successfully..." << endl;
 	active = true;
 	return cards * sizeindex;
 }
@@ -222,15 +388,15 @@ void VPBDriver::run(void)
         VPB_EVENT vpbevt;
         TrunkEvent event;
         VPBTrunk *trunk;
-        int port;
+        unsigned port;
         int cards = vpbivr.getCards();
-	char s[VPB_MAX_STR];
+//	char s[VPB_MAX_STR];
 
 	slog(Slog::levelInfo) << "vpb: event thread started..." << endl;
 	running = true;
 	active = true;
-
 	setCancel(cancelDeferred);
+
 	for(;;)
 	{
 		if(!active)
@@ -238,7 +404,7 @@ void VPBDriver::run(void)
 			Thread::sleep(1000);
 			continue;
 		}
-		
+
 		if(vpb_get_event_sync(&vpbevt, 0) != VPB_OK)
 		{
 			Thread::yield();
@@ -280,9 +446,10 @@ void VPBDriver::run(void)
                         event.parm.dtmf.digit = trunk->getDigit(vpbevt.data);
                         event.parm.dtmf.duration = 40;
                         break;
-		case VPB_DROP:
-			event.id = TRUNK_STOP_DISCONNECT;
-			break;
+                case VPB_DROP:
+			if(dropflag)
+	                        event.id = TRUNK_STOP_DISCONNECT;
+                        break;
                 case VPB_TIMEREXP:
                         event.id = TRUNK_TIMER_EXPIRED;
                         break;
@@ -292,13 +459,26 @@ void VPBDriver::run(void)
                 case VPB_VOXOFF:
                         event.id = TRUNK_AUDIO_STOP;
                         break;
-		                case VPB_PLAYEND:
+		case VPB_PLAYEND:
                 case VPB_RECORDEND:
                         event.id = TRUNK_AUDIO_IDLE;
                         break;
                 case VPB_DIALEND:
                         event.id = TRUNK_EXIT_STATE;
                         break;
+#ifdef	VPB_V12PCI
+		case VPB_STATION_OFFHOOK:
+			event.id = TRUNK_STATION_OFFHOOK;
+			break;
+		case VPB_STATION_ONHOOK:
+			event.id = TRUNK_STATION_ONHOOK;
+			break;
+#ifdef	VPB_STATION_FLASH
+		case VPB_STATION_FLASH:
+			event.id = TRUNK_STATION_FLASH;
+			break;
+#endif
+#endif
                 case VPB_CALLEND:
 			switch(vpbevt.data)
 			{
@@ -368,7 +548,7 @@ void VPBDriver::run(void)
 
 Trunk *VPBDriver::getTrunkPort(int id)
 {
-	if(id < 0 || id >= port_count)
+	if(id < 0 || id >= (int)port_count)
 		return NULL;
 
 	if(!ports)
