@@ -1,17 +1,17 @@
 // Copyright (C) 2000 Open Source Telecom Corporation.
-//  
+//
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 2 of the License, or
 // (at your option) any later version.
-// 
+//
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software 
+// along with this program; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 #include "driver.h"
@@ -22,9 +22,10 @@ namespace ost {
 #endif
 
 VPBPlay::VPBPlay(VPBTrunk *trunk, int h) :
-Service((Trunk *)trunk, keythreads.priAudio()), URLAudio()
+URLAudio(), Service((Trunk *)trunk, keythreads.priAudio())
 {
 	handle = h;
+	id = trunk->id;
 	reset = false;
 	vpbtr = trunk; // DR
 }
@@ -56,7 +57,7 @@ char *VPBPlay::getContinuation(void)
 		return NULL;
 retry:
 	fn = getPlayfile();
-	if(fn && data->play.mode == PLAY_MODE_ANY)
+	if(fn && (data->play.mode == PLAY_MODE_ANY || data->play.mode == PLAY_MODE_MOH))
 		if(!canAccess(fn))
 			goto retry;
 	return fn;
@@ -64,31 +65,49 @@ retry:
 
 void VPBPlay::initial(void)
 {
-	int codec;
+	int codec = VPB_MULAW;
 	char buffer[32];
 	char *fn;
 	struct stat ino;
 	struct tm *dt;
 	struct tm tbuf;
 	char *ann;
-	
+
 	trunk->getName(buffer);
+
+	if(tts)
+		if(!tts->synth(id, data))
+			Service::failure();
+
 retry:
 	fn = getPlayfile();
 	if(!fn)
 	{
-		if(data->play.mode == PLAY_MODE_ANY)
+		if(data->play.lock)
+		{
+			cachelock.unlock();
+			data->play.lock = false;
+		}
+		if(data->play.mode == PLAY_MODE_ANY || data->play.mode == PLAY_MODE_MOH)
 			success();
 		slog(Slog::levelError) << "no file" << endl;
 		Service::failure();
 	}
 	stat(fn, &ino);
-	open(fn);	
+	open(fn);
+	if(data->play.lock)
+	{
+		cachelock.unlock();
+		data->play.lock = false;
+	}
 	if(!isOpen())
 	{
 		if(data->play.mode == PLAY_MODE_ANY || data->play.mode == PLAY_MODE_ONE)
 			goto retry;
 
+		if(data->play.mode == PLAY_MODE_MOH)
+			goto retry;
+
 		slog(Slog::levelError) << fn << ": cannot open" << endl;
 		Service::failure();
 	}
@@ -108,7 +127,7 @@ retry:
 		trunk->setSymbol(SYM_ANNOTATION, ann);
 	else
 		trunk->setSymbol(SYM_ANNOTATION, "");
-	
+
 	switch(getEncoding())
 	{
 	case g721ADPCM:
@@ -151,7 +170,6 @@ void VPBPlay::run(void)
 	char name[33];
 	char buffer[bufsize];
 	Audio::Error status = Audio::errSuccess;
-	int len = 0;
 	float gain;
 
 	if(data->play.gain != 0.0)
@@ -195,6 +213,8 @@ void VPBPlay::run(void)
 
 bool VPBTrunk::playHandler(TrunkEvent *event)
 {
+	char *tag;
+
 	switch(event->id)
 	{
 	case TRUNK_STOP_STATE:
