@@ -140,6 +140,58 @@ bool VPBTrunk::sleepHandler(TrunkEvent *event)
 	return false;
 }
 
+bool VPBTrunk::threadHandler(TrunkEvent *event)
+{
+	switch(event->id)
+	{
+	case TRUNK_STOP_STATE:
+		stopped = true;
+		exitThread();
+		return true;
+	case TRUNK_EXIT_STATE:
+		endTimer();
+		stopServices();
+		handler = &VPBTrunk::stepHandler;
+		return true;
+	case TRUNK_TIMER_EXPIRED:
+                if(!stopped)
+                {
+                        setSymbol(SYM_ERROR, "thread-timeout");
+			if(!trunkSignal(TRUNK_SIGNAL_TIMEOUT))
+	                        trunkSignal(TRUNK_SIGNAL_ERROR);
+                        exitThread();
+                        stopped = true;
+                }
+                return true;
+	case TRUNK_SERVICE_FAILURE:
+		if(!stopped)
+		{
+			setSymbol(SYM_ERROR, "thread-error");
+			trunkSignal(TRUNK_SIGNAL_ERROR);
+			exitThread();
+			stopped = true;
+		}
+		return true;
+	case TRUNK_SERVICE_SUCCESS:
+		if(!stopped)
+		{
+			trunkSignal(TRUNK_SIGNAL_STEP);
+			stopped = true;
+			exitThread();
+		}
+		return true;
+	case TRUNK_ENTER_STATE:
+		stopped = false;
+		enterState("thread");
+		endTimer();
+		setDTMFDetect();
+		setTimer(data.sleep.wakeup);
+		thread->start();
+		return true;
+	}
+	return false;
+}
+
 #ifdef	CCXX_NAMESPACES
 };
 #endif
